local L = LibStub("AceLocale-3.0"):GetLocale("LootersAid")
local testmode = false
local fishing = {
	3567,4598,6256,6365,6366,6367,
	7996,12225,19022,19969,19970,
	19972,25978,33820,44050,45858,
	45991,45992,50287, 46106, 46069,
	46070, 46349}
local enchants = {846,2603,3269}
local GetSpellInfo = GetSpellInfo
local GetSpellLink = GetSpellLink
local auras  = {
	["WARRIOR"] = {
		["TANK1"] = {GetSpellInfo(71), 10, 0, "Spell"},				--Defensive Stance
		["DAMAGER1"] = {GetSpellInfo(2458), 30, "Spell"}			--Beserker Stance
	},
	["DEATHKNIGHT"] = { 
		["TANK1"] = {GetSpellInfo(48263), 57, 0, "Spell"},			--Blood Presence
		["DAMAGER1"] = {GetSpellInfo(48266), 1 , 18, "Talent"},		--Frost Presence
		["DAMAGER2"] = {GetSpellInfo(48265), 2, 16, "Talent"}		--Unholy Presence
	},
	["PALADIN"] = {
		["TANK1"] = {GetSpellInfo(25780), 16, 0, "Spell"},			--Righteous Fury
		["TANK2"] = {GetSpellInfo(465), 5, 0, "Spell"},				--Devotion Aura
		["DAMAGER1"] = {GetSpellInfo(7294), 26, 0, "Spell"},		--Retribution Aura
		["HEALER1"] = {GetSpellInfo(19746), 42, 0, "Spell"}			--Concentration Aura
	},
	["PRIEST"] = {},
	["SHAMAN"] = {},
	["DRUID"] = {},
	["ROGUE"] = {},
	["MAGE"] = {},
	["WARLOCK"] = {},
	["HUNTER"] = {
		["DAMAGER1"] = {GetSpellInfo(19506), 2, 11, "Talent"},		--Trueshot Aura
		["DAMAGER2"] = {GetSpellInfo(13165), 12, 0, "Spell"}		--Aspect of the Hawk
	}
}

local msg
local items = {}
local _G = _G
local GetInventoryItemLink = GetInventoryItemLink
local sfind = string.find

local function IsInstance()

	local inInstance, instanceType = IsInInstance()
	local ininstance = false

	if (inInstance ~= nil) then

		if (instanceType ~= "none") then

  			ininstance = true

		end

	end

  	return ininstance

end

local function fadeOut(self, elapsed)

	local fadetime = Kipper.frame.fadetime + elapsed

	if (Kipper.frame.fadetime == 0) then
	
		PlaySound("RaidWarning")
		
	end
	
	if (fadetime < 2) then

		Kipper.frame.fadetime = fadetime

	else

		Kipper.frame:SetScript("OnUpdate", nil)
		Kipper.frame.fadetime = 0
		UIFrameFadeOut(Kipper.frame, 4, 1, 0)

	end
	
end

local function showMsgs()

	for n, item in ipairs(items) do
	
		_G["Kipper_Msg" .. n]:SetText(item)
		
	end
	
	Kipper.frame:Show()
	Kipper.frame:SetScript("OnUpdate", fadeOut)
	
end

local function addMsg(link, linktype)

	if (table.getn(items) < 5) then
	
		if (linktype == 2) then
		
			table.insert(items, string.format(L["Warning! %s missing!"], link))
			
		else
		
			table.insert(items, string.format(L["Warning! %s equipped!"], link))
			
		end
		
	end
	
end

local function checkAura(aura)

	local found, hasaura

	if (aura[4] == "talent") then
		
		trank = select(GetTalentInfo(aura[2], aura[3]), 5) or 0

		if (trank > 0) then
			
			hasaura = UnitAura("player", aura[1]) or UnitBuff("player", aura[1])
				
			if (not hasaura) then
				
				found = 1
				addMsg(GetSpellLink(aura[1]), 2)
					
			end
				
		end
		
	else
	
		if (Kipper.LEVEL >= aura[2]) then
		
			hasaura = UnitBuff("player", aura[1]) or UnitAura("player", aura[1])
			
			if (not hasaura) then
			
				found = 1
				addMsg(GetSpellLink(aura[1]), 2)
				
			end
			
		end
		
	end
	
	return found
					
end

local function checkGear()

	local slot, itemid, ench
	local ilink, i
	local found, fishitem
	local result, aurainfo
	local hasaura, trank
	local foundb
	
	found = nil
	result = 1

	Kipper.frame:SetScript("OnUpdate", nil)
	
	--check class specific auras/buffs
	--currently only works for LFD dungeons
	if (Kipper.ROLE ~= "NONE") then
	
		aurainfo = auras[Kipper.CLASS][Kipper.ROLE .. "1"]
	
		if (aurainfo) then

			found = checkAura(aurainfo)
		
		end
	
		aurainfo = auras[Kipper.CLASS][Kipper.ROLE .. "2"]
		
		if (aurainfo) then

			foundb = checkAura(aurainfo)
			
		end
		
		found = found or foundb
	
	end
	
	--check inventory
	for slot = 0, 16 do

		ilink = GetInventoryItemLink("Player", slot)

		if (ilink) then

			_, _, _, _, itemid, ench = sfind(ilink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")


			for i, fishitem in ipairs(fishing) do

				result = itemid - fishitem


				if (result == 0) then

					found = 1
					addMsg(ilink)
	
				end
	
			end

			for i, enchant in pairs(enchants) do

				result = ench - enchant

				if (result == 0) then

					found = 1
					addMsg(ilink)
	
				end

			end

		end

	end

	if (found) then

		showMsgs()

	else
	
		Kipper.frame:Hide()
		
	end		

end

local function delay(self, elapsed)

	local waittime = Kipper.frame.wait + elapsed

	if (waittime < 5) then

		Kipper.frame.wait = waittime

	else

		checkGear()

	end
	
end

local function eventHandler(self, event, ...)

	local timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags = select(1, ...)
	local spellId, spellName, spellSchool = select(9, ...)

	if (event == "ZONE_CHANGED_NEW_AREA") then

		Kipper.frame:Hide()
		
		if (IsInstance() == true) then

			Kipper_Msg1:SetText("")
			Kipper_Msg2:SetText("")
			Kipper_Msg3:SetText("")
			Kipper_Msg4:SetText("")
			Kipper_Msg5:SetText("")
			items = {}
			Kipper.ROLE = UnitGroupRolesAssigned("player")
			if ((Kipper.ROLE == "NONE") and (testmode == true)) then Kipper.ROLE = "DAMAGER" end
			Kipper.frame.wait = 0
			Kipper.frame:Show()
			Kipper.frame:SetScript("OnUpdate", delay)

		end

	elseif (event == "PLAYER_LOGIN") then

		if (msg == nil) then

			msg1 = Kipper.frame:CreateFontString("Kipper_Msg1", "ARTWORK", "GameFontNormalLarge")
			msg1:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg1:SetPoint("CENTER", Kipper.frame, "CENTER", 0, 0)

			msg2 = Kipper.frame:CreateFontString("Kipper_Msg2", "ARTWORK", "GameFontNormalLarge")
			msg2:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg2:SetPoint("CENTER", msg1, "CENTER", 0, 32)
			
			msg3 = Kipper.frame:CreateFontString("Kipper_Msg3", "ARTWORK", "GameFontNormalLarge")
			msg3:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg3:SetPoint("CENTER", msg2, "CENTER", 0, 32)
			
			msg4 = Kipper.frame:CreateFontString("Kipper_Msg4", "ARTWORK", "GameFontNormalLarge")
			msg4:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg4:SetPoint("CENTER", msg3, "CENTER", 0, 32)
			
			msg5 = Kipper.frame:CreateFontString("Kipper_Msg5", "ARTWORK", "GameFontNormalLarge")
			msg5:SetFont("Fonts\\FRIZQT__.TTF",28)
			msg5:SetPoint("CENTER", msg4, "CENTER", 0, 32)
			
			_, Kipper.CLASS = UnitClass("player")
			Kipper.LEVEL = UnitLevel("player")
			
		end
		
	end

end

Kipper.frame = CreateFrame("Frame","Kipper",UIParent)
Kipper.frame:Hide()
Kipper.frame.fadetime = 0
Kipper.frame.wait = 0
Kipper.frame:SetFrameStrata("DIALOG")
Kipper.frame:SetHeight(200)
Kipper.frame:SetWidth(UIParent:GetWidth())
Kipper.frame:SetPoint("CENTER", UIParent, "CENTER", 0, 200)
Kipper.frame:RegisterEvent("PLAYER_LOGIN")
Kipper.frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
Kipper.frame:SetScript("OnEvent", eventHandler)